@@ -10,35 +10,40 @@ import re
 import os
 import json
 import logging
-import inspect
 import urllib
 from typing import (
     Union,
     Optional
 )
 
-import yaml
 import tornado.web
 import jsonschema
 from tornado.web import HTTPError
 from tornado.concurrent import Future
 
+import mockintosh
 from mockintosh.constants import PROGRAM, SUPPORTED_ENGINES, PYBARS, JINJA, SPECIAL_CONTEXT
 from mockintosh.exceptions import UnsupportedTemplateEngine
 from mockintosh.templating import TemplateRenderer
 from mockintosh.params import PathParam, HeaderParam, QueryStringParam
 from mockintosh.methods import _safe_path_split, _detect_engine
-from mockintosh.exceptions import UnrecognizedConfigFileFormat
+
+OPTIONS = 'options'
+ORIGIN = 'Origin'
+AC_REQUEST_HEADERS = 'Access-Control-Request-Headers'
+NON_PREFLIGHT_METHODS = ("GET", "HEAD", "POST", "DELETE", "PATCH", "PUT")
 
 
 class GenericHandler(tornado.web.RequestHandler):
 
-    def initialize(self, method, alternatives, _globals, definition_engine, interceptors):
-        self.alternatives = alternatives
+    def initialize(self, config_dir, methods, _globals, definition_engine, interceptors):
+        self.config_dir = config_dir
+        self.methods = {k.lower(): v for k, v in methods.items()}
+        self.alternatives = None
         self.globals = _globals
-        self.custom_method = method.lower()
         self.definition_engine = definition_engine
         self.interceptors = interceptors
+        self.is_options = False
 
         self.special_request = self.build_special_request()
         self.default_context = {
@@ -47,6 +52,11 @@ class GenericHandler(tornado.web.RequestHandler):
         self.custom_context = {}
 
     def super_verb(self, *args):
+        self.set_default_headers()
+
+        if not self.__class__.__name__ == 'ErrorHandler' and not self.is_options:
+            self.dynamic_unimplemented_method_guard()
+
         try:
             _id, response, params, context = self.match_alternative()
         except TypeError:
@@ -60,10 +70,12 @@ class GenericHandler(tornado.web.RequestHandler):
         self.determine_status_code()
         self.determine_headers()
         self.log_request()
-        self.dynamic_unimplemented_method_guard()
         self.rendered_body = self.render_template()
+        if self.rendered_body is None:
+            return
         self.special_response = self.build_special_response()
-        self.write(self.rendered_body)
+        if self.should_write():
+            self.write(self.rendered_body)
 
     def get(self, *args):
         self.super_verb(*args)
@@ -84,6 +96,7 @@ class GenericHandler(tornado.web.RequestHandler):
         self.super_verb(*args)
 
     def options(self, *args):
+        self.is_options = True
         self.super_verb(*args)
 
     def populate_context(self, *args):
@@ -99,7 +112,7 @@ class GenericHandler(tornado.web.RequestHandler):
         self.analyze_query_string()
 
     def dynamic_unimplemented_method_guard(self):
-        if self.custom_method != inspect.stack()[2][3]:
+        if self.request.method.lower() not in self.methods:
             self._unimplemented_method()
 
     def log_request(self):
@@ -117,7 +130,6 @@ class GenericHandler(tornado.web.RequestHandler):
 
     def render_template(self):
         source_text = None
-        response = None
 
         is_response_str = isinstance(self.custom_response, str)
         template_engine = _detect_engine(self.custom_response, 'response', default=self.definition_engine)
@@ -128,19 +140,21 @@ class GenericHandler(tornado.web.RequestHandler):
             source_text = ''
             is_response_str = True
         elif 'body' in self.custom_response:
-            body = self.custom_response['body']
-            if len(body) > 1 and body[0] == '@' and os.path.isfile(body[1:]):
-                template_path = body[1:]
-                with open(template_path, 'r') as file:
-                    logging.info('Reading template file from path: %s' % template_path)
-                    source_text = file.read()
-                    logging.debug('Template file text: %s' % source_text)
-            else:
-                is_response_str = True
-                source_text = body
+            source_text = self.custom_response['body']
+            is_response_str = True
         else:
             return ''
 
+        if len(source_text) > 1 and source_text[0] == '@':
+            template_path = self.resolve_relative_path(source_text)
+            if template_path is None:
+                return None
+            with open(template_path, 'r') as file:
+                logging.info('Reading template file from path: %s' % template_path)
+                source_text = file.read()
+                logging.debug('Template file text: %s' % source_text)
+            is_response_str = False
+
         compiled = None
         if 'useTemplating' in self.custom_response and self.custom_response['useTemplating'] is False:
             compiled = source_text
@@ -167,20 +181,7 @@ class GenericHandler(tornado.web.RequestHandler):
 
         logging.debug('Render output: %s' % compiled)
 
-        if is_response_str:
-            return compiled
-        else:
-            try:
-                response = yaml.safe_load(compiled)
-                logging.info('Template is a valid YAML.')
-            except (yaml.scanner.ScannerError, yaml.parser.ParserError) as e:
-                raise UnrecognizedConfigFileFormat(
-                    'Template is neither a JSON nor a YAML!',
-                    compiled,
-                    str(e)
-                )
-
-        return response
+        return compiled
 
     def build_special_request(self):
         request = Request()
@@ -206,17 +207,17 @@ class GenericHandler(tornado.web.RequestHandler):
 
         # Query String
         for key, value in self.request.query_arguments.items():
-            request.queryString[key] = [x.decode('utf-8') for x in value]
+            request.queryString[key] = [self.decoder(x) for x in value]
             if len(request.queryString[key]) == 1:
                 request.queryString[key] = request.queryString[key][0]
 
         # Body
-        request.body = self.request.body.decode('utf-8')
+        request.body = self.decoder(self.request.body)
         request.files = self.request.files
 
         # Form Data
         for key, value in self.request.body_arguments.items():
-            request.formData[key] = [x.decode('utf-8') for x in value]
+            request.formData[key] = [self.decoder(x) for x in value]
             if len(request.formData[key]) == 1:
                 request.formData[key] = request.formData[key][0]
 
@@ -240,7 +241,8 @@ class GenericHandler(tornado.web.RequestHandler):
         self._write_buffer = []
         if self.rendered_body is None:
             self.rendered_body = ''
-        self.write(self.rendered_body)
+        if self.should_write():
+            self.write(self.rendered_body)
 
     def determine_status_code(self):
         status_code = None
@@ -325,6 +327,13 @@ class GenericHandler(tornado.web.RequestHandler):
                     self.set_header(key, value)
 
     def match_alternative(self):
+        if self.should_cors():
+            self.respond_cors()
+            return
+
+        if not self.__class__.__name__ == 'ErrorHandler':
+            self.alternatives = self.methods[self.request.method.lower()]
+
         response = None
         params = None
         context = None
@@ -374,20 +383,33 @@ class GenericHandler(tornado.web.RequestHandler):
             if 'body' in alternative:
                 if 'schema' in alternative['body']:
                     json_schema = alternative['body']['schema']
+                    if isinstance(json_schema, str) and len(json_schema) > 1 and json_schema[0] == '@':
+                        json_schema_path = self.resolve_relative_path(json_schema)
+                        with open(json_schema_path, 'r') as file:
+                            logging.info('Reading JSON schema file from path: %s' % json_schema_path)
+                            json_schema = json.load(file)
+                            logging.debug('JSON schema: %s' % json_schema)
                     body = self.request.body.decode('utf-8')
                     json_data = None
 
-                    try:
-                        json_data = json.loads(body)
-                    except json.decoder.JSONDecodeError:
-                        self.set_status(404)
-                        self.finish()
+                    if body and json_schema:
+                        try:
+                            json_data = json.loads(body)
+                        except json.decoder.JSONDecodeError:
+                            fail = True
+                            break
 
-                    try:
-                        jsonschema.validate(instance=json_data, schema=json_schema)
-                    except jsonschema.exceptions.ValidationError:
-                        self.set_status(404)
-                        self.finish()
+                    if json_schema:
+                        try:
+                            jsonschema.validate(instance=json_data, schema=json_schema)
+                        except jsonschema.exceptions.ValidationError:
+                            fail = True
+                            break
+
+            if self.should_cors():
+                self.respond_cors()
+            else:
+                self.set_cors_headers()
 
             _id = alternative['id']
             response = alternative['response']
@@ -403,13 +425,79 @@ class GenericHandler(tornado.web.RequestHandler):
             interceptor(self.special_request, self.special_response)
 
     def finish(self, chunk: Optional[Union[str, bytes, dict]] = None) -> "Future[None]":
-        if not self._status_code == 500:
+        if self._status_code not in (204, 500):
             if not hasattr(self, 'special_response'):
                 self.special_response = self.build_special_response()
             self.trigger_interceptors()
-            self.update_response()
+            if self.interceptors:
+                self.update_response()
         super().finish(chunk)
 
+    def should_write(self):
+        return not hasattr(self, 'custom_response') or (
+            'body' in self.custom_response or isinstance(self.custom_response, str)
+        )
+
+    def decoder(self, string):
+        try:
+            return string.decode('utf-8')
+        except UnicodeDecodeError:
+            return string.decode('latin-1')
+
+    def resolve_relative_path(self, source_text):
+        relative_path = None
+        orig_relative_path = source_text[1:]
+        error_msg = 'External template file \'%s\' couldn\'t be accessed or found!' % orig_relative_path
+        if orig_relative_path[0] == '/':
+            orig_relative_path = orig_relative_path[1:]
+        relative_path = os.path.join(self.config_dir, orig_relative_path)
+        if not os.path.isfile(relative_path):
+            self.send_error(403, message=error_msg)
+            return None
+        relative_path = os.path.abspath(relative_path)
+        if not relative_path.startswith(self.config_dir):
+            self.send_error(403, message=error_msg)
+            return None
+
+        return relative_path
+
+    def write_error(self, status_code: int, **kwargs) -> None:
+        if 'message' in kwargs and kwargs['message']:
+            self.finish(kwargs['message'])
+        else:
+            self.finish()
+
+    def respond_cors(self):
+        if ORIGIN not in self.request.headers._dict:
+            # Invalid CORS preflight request
+            self.set_status(404)
+            self.finish()
+
+        self.set_cors_headers()
+        self.set_status(204)
+        self.finish()
+
+    def set_cors_headers(self):
+        if ORIGIN in self.request.headers._dict:
+            self.set_header('access-control-allow-methods', 'DELETE, GET, HEAD, OPTIONS, PATCH, POST, PUT')
+            origin = self.request.headers.get(ORIGIN)
+            self.set_header('access-control-allow-origin', origin)
+
+            if AC_REQUEST_HEADERS in self.request.headers._dict:
+                ac_request_headers = self.request.headers.get(AC_REQUEST_HEADERS)
+                self.set_header('access-control-allow-headers', ac_request_headers)
+
+    def set_default_headers(self):
+        self.set_header('Server', '%s/%s' % (
+            PROGRAM.capitalize(),
+            mockintosh.__version__
+        ))
+
+    def should_cors(self):
+        return not self.__class__.__name__ == 'ErrorHandler' and (
+            self.is_options and self.request.method.lower() not in self.methods.keys()
+        )
+
 
 class Request():
 
