@@ -30,7 +30,7 @@ def _detect_engine(data, context='config', default=PYBARS):
         data['templatingEngine'].lower() in (JINJA.lower(), SHORT_JINJA)
     ):
         template_engine = JINJA
-    logging.info('Templating engine (%s) is: %s' % (context, template_engine))
+    logging.debug('Templating engine (%s) is: %s' % (context, template_engine))
     return template_engine
 
 
