@@ -24,6 +24,7 @@ class HttpServer():
         self.globals = self.definition.data['globals'] if 'globals' in self.definition.data else {}
         self.debug = debug
         self.interceptors = interceptors
+        self.services_log = []
         self.load()
 
     def load(self):
@@ -44,6 +45,11 @@ class HttpServer():
                 if 'hostname' not in service:
                     app.listen(service['port'])
                     logging.info('Will listen port number: %d' % service['port'])
+                    self.services_log.append('Serving at http://%s:%s%s' % (
+                        'localhost',
+                        service['port'],
+                        ' the mock for %s' % service['comment'] if 'comment' in service else ''
+                    ))
                 else:
                     rules.append(
                         Rule(HostMatches(service['hostname']), app)
@@ -54,7 +60,13 @@ class HttpServer():
                         service['hostname'],
                         service['port']
                     ))
-                logging.info('Finished registering: %s' % service['comment'])
+                    self.services_log.append('Serving at http://%s:%s%s' % (
+                        service['hostname'],
+                        service['port'],
+                        ' the mock for %s' % service['comment'] if 'comment' in service else ''
+                    ))
+                if 'comment' in service:
+                    logging.info('Finished registering: %s' % service['comment'])
 
             if rules:
                 router = RuleRouter(rules)
@@ -67,7 +79,7 @@ class HttpServer():
         for endpoint in endpoints:
             if 'method' not in endpoint:
                 endpoint['method'] = 'GET'
-            identifier = '%s %s' % (endpoint['method'], endpoint['path'])
+            identifier = endpoint['path']
             extracted_parts = {}
             for key in endpoint:
                 if key in ('method', 'path', 'priority'):
@@ -75,14 +87,15 @@ class HttpServer():
                 extracted_parts[key] = endpoint[key]
                 if 'id' not in extracted_parts:
                     extracted_parts['id'] = None
-            if identifier in new_endpoints:
-                new_endpoints[identifier]['alternatives'].append(extracted_parts)
-            else:
+            if identifier not in new_endpoints:
                 new_endpoints[identifier] = {}
-                new_endpoints[identifier]['method'] = endpoint['method']
                 new_endpoints[identifier]['path'] = endpoint['path']
                 new_endpoints[identifier]['priority'] = endpoint['priority']
-                new_endpoints[identifier]['alternatives'] = [extracted_parts]
+                new_endpoints[identifier]['methods'] = {}
+            if endpoint['method'] not in new_endpoints[identifier]['methods']:
+                new_endpoints[identifier]['methods'][endpoint['method']] = [extracted_parts]
+            else:
+                new_endpoints[identifier]['methods'][endpoint['method']].append(extracted_parts)
         return new_endpoints.values()
 
     def run(self):
@@ -92,6 +105,9 @@ class HttpServer():
             parent_pid = os.getppid()
             os.kill(parent_pid, signal.SIGALRM)
 
+        for service_log in list(set(self.services_log)):
+            logging.info(service_log)
+
         logging.info('Mock server is ready!')
         tornado.ioloop.IOLoop.current().start()
 
@@ -105,14 +121,15 @@ class HttpServer():
                     endpoint['path'],
                     GenericHandler,
                     dict(
-                        method=endpoint['method'],
-                        alternatives=endpoint['alternatives'],
+                        config_dir=self.definition.source_dir,
+                        methods=endpoint['methods'],
                         _globals=_globals,
                         definition_engine=self.definition.template_engine,
                         interceptors=self.interceptors
                     )
                 )
             )
-            logging.info('Registered endpoint: %s %s' % (endpoint['method'].upper(), endpoint['path']))
-            logging.debug('with alternatives:\n%s' % endpoint['alternatives'])
+            for method, alternatives in endpoint['methods'].items():
+                logging.info('Registered endpoint: %s %s' % (method.upper(), endpoint['path']))
+                logging.debug('with alternatives:\n%s' % alternatives)
         return Application(endpoint_handlers, debug=debug, interceptors=self.interceptors)
